@@ -21,9 +21,10 @@
 #include <kcalcore/freebusyurlstore.h>
 #include <kdebug.h>
 #include <qbuffer.h>
-#include <kio/netaccess.h>
 #include "kolabformat/errorhandler.h"
 
+
+
 namespace Kolab {
     namespace Conversion {
         
@@ -327,10 +328,12 @@ std::string fromPicture(const KABC::Picture &pic, std::string &mimetype)
         }
     } else if ( !pic.url().isEmpty() ) {
         QString tmpFile;
-        if ( KIO::NetAccess::download( pic.url(), tmpFile, 0 /*no widget known*/ ) ) {
-            img.load( tmpFile );
-            KIO::NetAccess::removeTempFile( tmpFile );
-        }
+        kWarning() << "external pictures are currently not supported";
+        //FIXME add kio support to libcalendaring or use libcurl
+//         if ( KIO::NetAccess::download( pic.url(), tmpFile, 0 /*no widget known*/ ) ) {
+//             img.load( tmpFile );
+//             KIO::NetAccess::removeTempFile( tmpFile );
+//         }
     }
     if (img.isNull()) {
         Error() << "invalid picture";
@@ -362,7 +365,7 @@ KABC::Picture toPicture(const std::string &data, const std::string &mimetype) {
     if (logo.isEmpty()) {
         Error() << "failed to read picture";
     }
-    logo.setType(QString::fromStdString(mimetype));
+    logo.setType(fromStdString(mimetype));
     return logo;
 }
 
@@ -389,29 +392,29 @@ std::string getCustom(const std::string &id, T &object)
 KABC::Addressee toKABC(const Kolab::Contact &contact)
 {
   KABC::Addressee addressee;
-  addressee.setUid(QString::fromStdString(contact.uid()));
+  addressee.setUid(fromStdString(contact.uid()));
   addressee.setCategories(toStringList(contact.categories()));
-  //addressee.setName(QString::fromStdString(contact.name()));//This one is only for compatiblity (and results in a non-existing name property)
-  addressee.setFormattedName(QString::fromStdString(contact.name())); //This on corresponds to fn
+  //addressee.setName(fromStdString(contact.name()));//This one is only for compatiblity (and results in a non-existing name property)
+  addressee.setFormattedName(fromStdString(contact.name())); //This on corresponds to fn
   
   const Kolab::NameComponents &nc = contact.nameComponents();
   if (!nc.surnames().empty()) {
-      addressee.setFamilyName(QString::fromStdString(nc.surnames().front()));
+      addressee.setFamilyName(fromStdString(nc.surnames().front()));
   }
   if (!nc.given().empty()) {
-      addressee.setGivenName(QString::fromStdString(nc.given().front()));
+      addressee.setGivenName(fromStdString(nc.given().front()));
   }
   if (!nc.additional().empty()) {
-      addressee.setAdditionalName(QString::fromStdString(nc.additional().front()));
+      addressee.setAdditionalName(fromStdString(nc.additional().front()));
   }
   if (!nc.prefixes().empty()) {
-      addressee.setPrefix(QString::fromStdString(nc.prefixes().front()));
+      addressee.setPrefix(fromStdString(nc.prefixes().front()));
   }
   if (!nc.suffixes().empty()) {
-      addressee.setSuffix(QString::fromStdString(nc.suffixes().front()));
+      addressee.setSuffix(fromStdString(nc.suffixes().front()));
   }
   
-  addressee.setNote(QString::fromStdString(contact.note()));
+  addressee.setNote(fromStdString(contact.note()));
   
   addressee.setSecrecy(KABC::Secrecy::Public); //We don't have any privacy setting in xCard
   
@@ -420,10 +423,10 @@ KABC::Addressee toKABC(const Kolab::Contact &contact)
   
   if (!contact.emailAddresses().empty()) {
       addressee.setEmails(toStringList(contact.emailAddresses()));
-      if ((contact.emailAddressPreferredIndex() >= 0) && (contact.emailAddressPreferredIndex() < contact.emailAddresses().size())) {
-          preferredEmail = QString::fromStdString(contact.emailAddresses().at(contact.emailAddressPreferredIndex()));
+      if ((contact.emailAddressPreferredIndex() >= 0) && (contact.emailAddressPreferredIndex() < static_cast<int>(contact.emailAddresses().size()))) {
+          preferredEmail = fromStdString(contact.emailAddresses().at(contact.emailAddressPreferredIndex()));
       } else {
-          preferredEmail = QString::fromStdString(contact.emailAddresses().at(0));
+          preferredEmail = fromStdString(contact.emailAddresses().at(0));
       }
       addressee.insertEmail(preferredEmail, true);
   }
@@ -432,26 +435,26 @@ KABC::Addressee toKABC(const Kolab::Contact &contact)
       if (preferredEmail.isEmpty()) {
           Error() << "f/b url is set but no email address available, skipping";
       } else  {
-        KCalCore::FreeBusyUrlStore::self()->writeUrl( preferredEmail, QString::fromStdString(contact.freeBusyUrl()) );
+        KCalCore::FreeBusyUrlStore::self()->writeUrl( preferredEmail, fromStdString(contact.freeBusyUrl()) );
         KCalCore::FreeBusyUrlStore::self()->sync();
       }
   }
   
   if (!contact.nickNames().empty()) {
-      addressee.setNickName(QString::fromStdString(contact.nickNames().at(0))); //TODO support multiple
+      addressee.setNickName(fromStdString(contact.nickNames().at(0))); //TODO support multiple
   }
   
   if (contact.bDay().isValid()) {
       addressee.setBirthday(toDate(contact.bDay()).dateTime());
   }
   if (!contact.titles().empty()) {
-      addressee.setTitle(QString::fromStdString(contact.titles().at(0))); //TODO support multiple
+      addressee.setTitle(fromStdString(contact.titles().at(0))); //TODO support multiple
   }
   if (!contact.urls().empty()) {
-      addressee.setUrl(QString::fromStdString(contact.urls().at(0).url())); //TODO support multiple
+      addressee.setUrl(fromStdString(contact.urls().at(0).url())); //TODO support multiple
       foreach(const Kolab::Url &u, contact.urls()) {
           if (u.type() == Kolab::Url::Blog) {
-              addressee.insertCustom("KADDRESSBOOK", "BlogFeed", QString::fromStdString(u.url()));
+              addressee.insertCustom("KADDRESSBOOK", "BlogFeed", fromStdString(u.url()));
           }
       }
   }
@@ -461,13 +464,13 @@ KABC::Addressee toKABC(const Kolab::Contact &contact)
       //Storing only a const reference leads to segfaults. No idea why.
       const Kolab::Affiliation aff = contact.affiliations().at(0); //TODO support multiple
       if (!aff.organisation().empty()) {
-          addressee.setOrganization(QString::fromStdString(aff.organisation()));
+          addressee.setOrganization(fromStdString(aff.organisation()));
       }
       if (!aff.organisationalUnits().empty()) {
-          addressee.setDepartment(QString::fromStdString(aff.organisationalUnits().at(0))); //TODO support multiple
+          addressee.setDepartment(fromStdString(aff.organisationalUnits().at(0))); //TODO support multiple
       }
       if (!aff.roles().empty()) {
-          addressee.setRole(QString::fromStdString(aff.roles().at(0))); //TODO support multiple
+          addressee.setRole(fromStdString(aff.roles().at(0))); //TODO support multiple
       }
       if (!aff.logo().empty()) {
           addressee.setLogo(toPicture(aff.logo(), aff.logoMimetype()));
@@ -478,24 +481,24 @@ KABC::Addressee toKABC(const Kolab::Contact &contact)
               continue;
           }
           if (related.relationTypes() & Kolab::Related::Assistant) {
-              addressee.insertCustom(QLatin1String("KADDRESSBOOK"), QLatin1String("X-AssistantsName"), QString::fromStdString(related.text()));
+              addressee.insertCustom(QLatin1String("KADDRESSBOOK"), QLatin1String("X-AssistantsName"), fromStdString(related.text()));
           }
           if (related.relationTypes() & Kolab::Related::Manager) {
-              addressee.insertCustom(QLatin1String("KADDRESSBOOK"), QLatin1String("X-ManagersName"), QString::fromStdString(related.text()));
+              addressee.insertCustom(QLatin1String("KADDRESSBOOK"), QLatin1String("X-ManagersName"), fromStdString(related.text()));
           }
       }
       foreach(const Kolab::Address &address, aff.addresses()) {
-          addressee.insertCustom(QLatin1String("KADDRESSBOOK"), QLatin1String("X-Office"), QString::fromStdString(address.label())); //TODO support proper addresses
+          addressee.insertCustom(QLatin1String("KADDRESSBOOK"), QLatin1String("X-Office"), fromStdString(address.label())); //TODO support proper addresses
       }
   }
   const std::string &prof = getCustom("X-Profession", contact);
   if (!prof.empty()) {
-    addressee.insertCustom(QLatin1String("KADDRESSBOOK"), QLatin1String("X-Profession"), QString::fromStdString(prof));
+    addressee.insertCustom(QLatin1String("KADDRESSBOOK"), QLatin1String("X-Profession"), fromStdString(prof));
   }
   
   const std::string &adrBook = getCustom("X-AddressBook", contact);
   if (!adrBook.empty()) {
-      addressee.insertCustom(QLatin1String("KADDRESSBOOK"), QLatin1String("X-AddressBook"), QString::fromStdString(prof));
+      addressee.insertCustom(QLatin1String("KADDRESSBOOK"), QLatin1String("X-AddressBook"), fromStdString(prof));
   }
   
   if (!contact.photo().empty()) {
@@ -509,7 +512,7 @@ KABC::Addressee toKABC(const Kolab::Contact &contact)
           if (index == contact.telephonesPreferredIndex()) {
               pref = true;
           }
-          KABC::PhoneNumber number(QString::fromStdString(tel.number()), toPhoneType(tel.types(), pref));
+          KABC::PhoneNumber number(fromStdString(tel.number()), toPhoneType(tel.types(), pref));
           index++;
           addressee.insertPhoneNumber(number);
       }
@@ -523,12 +526,12 @@ KABC::Addressee toKABC(const Kolab::Contact &contact)
               pref = true;
           }
           KABC::Address adr(toAddressType(a.types(), pref));          
-          adr.setLabel(QString::fromStdString(a.label()));
-          adr.setStreet(QString::fromStdString(a.street()));
-          adr.setLocality(QString::fromStdString(a.locality()));
-          adr.setRegion(QString::fromStdString(a.region()));
-          adr.setPostalCode(QString::fromStdString(a.code()));
-          adr.setCountry(QString::fromStdString(a.country()));
+          adr.setLabel(fromStdString(a.label()));
+          adr.setStreet(fromStdString(a.street()));
+          adr.setLocality(fromStdString(a.locality()));
+          adr.setRegion(fromStdString(a.region()));
+          adr.setPostalCode(fromStdString(a.code()));
+          adr.setCountry(fromStdString(a.country()));
           
           index++;
           addressee.insertAddress(adr);
@@ -540,7 +543,7 @@ KABC::Addressee toKABC(const Kolab::Contact &contact)
   }
   
   if (!contact.imAddresses().empty()) {
-      addressee.insertCustom(QLatin1String("KADDRESSBOOK"), QLatin1String("X-IMAddress"),QString::fromStdString(contact.imAddresses()[0])); //TODO support multiple
+      addressee.insertCustom(QLatin1String("KADDRESSBOOK"), QLatin1String("X-IMAddress"),fromStdString(contact.imAddresses()[0])); //TODO support multiple
   }
   
   if (!contact.relateds().empty()) {
@@ -550,7 +553,7 @@ KABC::Addressee toKABC(const Kolab::Contact &contact)
               continue;
           }
           if (rel.relationTypes() & Kolab::Related::Spouse) {
-              addressee.insertCustom(QLatin1String("KADDRESSBOOK"), QLatin1String("X-SpousesName"),QString::fromStdString(rel.text())); //TODO support multiple
+              addressee.insertCustom(QLatin1String("KADDRESSBOOK"), QLatin1String("X-SpousesName"),fromStdString(rel.text())); //TODO support multiple
           } else {
               Warning() << "relation not supported";
               continue;
@@ -566,32 +569,32 @@ Kolab::Contact fromKABC(const KABC::Addressee &addressee)
     int prefNum = -1;
     int prefCounter = -1;
     Kolab::Contact c;
-    c.setUid(addressee.uid().toStdString());
+    c.setUid(toStdString(addressee.uid()));
     c.setCategories(fromStringList(addressee.categories()));
-    c.setName(addressee.formattedName().toStdString());
-    
+    c.setName(toStdString(addressee.formattedName()));
     Kolab::NameComponents nc;
-    nc.setSurnames(std::vector<std::string>() << addressee.familyName().toStdString());
-    nc.setGiven(std::vector<std::string>() << addressee.givenName().toStdString());
-    nc.setAdditional(std::vector<std::string>() << addressee.additionalName().toStdString());
-    nc.setPrefixes(std::vector<std::string>() << addressee.prefix().toStdString());
-    nc.setSuffixes(std::vector<std::string>() << addressee.suffix().toStdString());
+    nc.setSurnames(std::vector<std::string>() << toStdString(addressee.familyName()));
+    nc.setGiven(std::vector<std::string>() << toStdString(addressee.givenName()));
+    nc.setAdditional(std::vector<std::string>() << toStdString(addressee.additionalName()));
+    nc.setPrefixes(std::vector<std::string>() << toStdString(addressee.prefix()));
+    nc.setSuffixes(std::vector<std::string>() << toStdString(addressee.suffix()));
     c.setNameComponents(nc);
     
-    c.setNote(addressee.note().toStdString());
+    c.setNote(toStdString(addressee.note()));
     
     if ( !addressee.preferredEmail().isEmpty() ) {    
-        c.setFreeBusyUrl(KCalCore::FreeBusyUrlStore::self()->readUrl( addressee.preferredEmail() ).toStdString());
+        c.setFreeBusyUrl(toStdString(KCalCore::FreeBusyUrlStore::self()->readUrl( addressee.preferredEmail() )));
     }
 
     if (!addressee.title().isEmpty()) {
-        c.setTitles(std::vector<std::string>() << addressee.title().toStdString());
+        c.setTitles(std::vector<std::string>() << toStdString(addressee.title()));
     }
     
     Kolab::Affiliation businessAff;
-    businessAff.setOrganisation(addressee.organization().toStdString());
+    businessAff.setOrganisation(toStdString(addressee.organization()));
     if (!addressee.department().isEmpty()) {
-        businessAff.setOrganisationalUnits(std::vector<std::string>() << addressee.department().toStdString());
+        Debug() << addressee.department() << addressee.department().toLatin1() << addressee.department().toUtf8();
+        businessAff.setOrganisationalUnits(std::vector<std::string>() << toStdString(addressee.department()));
     }
 
     if (!addressee.logo().isEmpty()) {
@@ -600,24 +603,24 @@ Kolab::Contact fromKABC(const KABC::Addressee &addressee)
         businessAff.setLogo(logo, logoMimetype);
     }
     if (!addressee.role().isEmpty()) {
-        businessAff.setRoles(std::vector<std::string>() << addressee.role().toStdString());
+        businessAff.setRoles(std::vector<std::string>() << toStdString(addressee.role()));
     }
     const QString &office = addressee.custom(QLatin1String("KADDRESSBOOK"), QLatin1String("X-Office"));
     if (!office.isEmpty()) {
         Kolab::Address a;
         a.setTypes(Kolab::Address::Work);
-        a.setLabel(addressee.custom(QLatin1String("KADDRESSBOOK"), QLatin1String("X-Office")).toStdString());
+        a.setLabel(toStdString(addressee.custom(QLatin1String("KADDRESSBOOK"), QLatin1String("X-Office"))));
         businessAff.setAddresses(std::vector<Kolab::Address>() << a);
     }
     
     std::vector<Kolab::Related> relateds;
     const QString &manager = addressee.custom(QLatin1String("KADDRESSBOOK"), QLatin1String("X-ManagersName"));
     if (!manager.isEmpty()) {
-        relateds.push_back(Kolab::Related(Kolab::Related::Text, manager.toStdString(), Kolab::Related::Manager));
+        relateds.push_back(Kolab::Related(Kolab::Related::Text, toStdString(manager), Kolab::Related::Manager));
     }
     const QString &assistant = addressee.custom(QLatin1String("KADDRESSBOOK"), QLatin1String("X-AssistantsName"));
     if (!assistant.isEmpty()) {
-        relateds.push_back(Kolab::Related(Kolab::Related::Text, assistant.toStdString(), Kolab::Related::Assistant));
+        relateds.push_back(Kolab::Related(Kolab::Related::Text, toStdString(assistant), Kolab::Related::Assistant));
     }
     if (!relateds.empty()) {
         businessAff.setRelateds(relateds);
@@ -626,11 +629,11 @@ Kolab::Contact fromKABC(const KABC::Addressee &addressee)
     
     std::vector<Kolab::Url> urls;
     if (!addressee.url().isEmpty()) {
-        urls.push_back(Kolab::Url(addressee.url().url().toStdString()));
+        urls.push_back(Kolab::Url(toStdString(addressee.url().url())));
     }   
     const QString &blogUrl = addressee.custom(QLatin1String("KADDRESSBOOK"), QLatin1String("BlogFeed"));
     if (!blogUrl.isEmpty()) {
-        urls.push_back(Kolab::Url(blogUrl.toStdString(), Kolab::Url::Blog));
+        urls.push_back(Kolab::Url(toStdString(blogUrl), Kolab::Url::Blog));
     }
     c.setUrls(urls);
     
@@ -645,23 +648,23 @@ Kolab::Contact fromKABC(const KABC::Addressee &addressee)
         if (pref) {
             prefNum = prefCounter;
         }
-        adr.setLabel(a.label().toStdString());
-        adr.setStreet(a.street().toStdString());
-        adr.setLocality(a.locality().toStdString());
-        adr.setRegion(a.region().toStdString());
-        adr.setCode(a.postalCode().toStdString());
-        adr.setCountry(a.country().toStdString());
+        adr.setLabel(toStdString(a.label()));
+        adr.setStreet(toStdString(a.street()));
+        adr.setLocality(toStdString(a.locality()));
+        adr.setRegion(toStdString(a.region()));
+        adr.setCode(toStdString(a.postalCode()));
+        adr.setCountry(toStdString(a.country()));
         addresses.push_back(adr);
     }
     c.setAddresses(addresses, prefNum);
     
     if (!addressee.nickName().isEmpty()) {
-        c.setNickNames(std::vector<std::string>() << addressee.nickName().toStdString());
+        c.setNickNames(std::vector<std::string>() << toStdString(addressee.nickName()));
     }
     
     const QString &spouse = addressee.custom(QLatin1String("KADDRESSBOOK"), QLatin1String("X-SpousesName"));
     if (!spouse.isEmpty()) {
-        c.setRelateds(std::vector<Kolab::Related>() << Kolab::Related(Kolab::Related::Text, spouse.toStdString(), Kolab::Related::Spouse));
+        c.setRelateds(std::vector<Kolab::Related>() << Kolab::Related(Kolab::Related::Text, toStdString(spouse), Kolab::Related::Spouse));
     }
     c.setBDay(fromDate(KDateTime(addressee.birthday(), KDateTime::ClockTime)));
     c.setAnniversary(fromDate(KDateTime(QDate::fromString( addressee.custom(QLatin1String("KADDRESSBOOK"), QLatin1String("X-Anniversary")), Qt::ISODate ), KDateTime::ClockTime)));
@@ -679,7 +682,7 @@ Kolab::Contact fromKABC(const KABC::Addressee &addressee)
     prefCounter = -1;
     foreach (const KABC::PhoneNumber &n, addressee.phoneNumbers()) {
         Kolab::Telephone p;
-        p.setNumber(n.number().toStdString());
+        p.setNumber(toStdString(n.number()));
         bool pref = false;
         p.setTypes(fromPhoneType(n.type(), pref));
         prefCounter++;
@@ -692,7 +695,7 @@ Kolab::Contact fromKABC(const KABC::Addressee &addressee)
     
     const QString &imAddress = addressee.custom(QLatin1String("KADDRESSBOOK"), QLatin1String("X-IMAddress"));
     if (!imAddress.isEmpty()) {
-        c.setIMaddresses(std::vector<std::string>() << imAddress.toStdString(), 0);
+        c.setIMaddresses(std::vector<std::string>() << toStdString(imAddress), 0);
     }
     
     int prefEmail = -1;
@@ -771,11 +774,11 @@ Kolab::Contact fromKABC(const KABC::Addressee &addressee)
     
     //FIXME the keys are most certainly worng, look at cryptopageplugin.cpp
     std::vector<Kolab::Key> keys;
-    const std::string &pgpkey = addressee.custom( "KADDRESSBOOK", "OPENPGPFP" ).toStdString();
+    const std::string &pgpkey = toStdString(addressee.custom( "KADDRESSBOOK", "OPENPGPFP" ));
     if (!pgpkey.empty()) {
         keys.push_back(Kolab::Key(pgpkey, Kolab::Key::PGP));
     }
-    const std::string &smimekey = addressee.custom( "KADDRESSBOOK", "SMIMEFP" ).toStdString();
+    const std::string &smimekey = toStdString(addressee.custom( "KADDRESSBOOK", "SMIMEFP" ));
     if (!smimekey.empty()) {
         keys.push_back(Kolab::Key(smimekey, Kolab::Key::PKCS7_MIME));
     }
@@ -786,12 +789,12 @@ Kolab::Contact fromKABC(const KABC::Addressee &addressee)
         Warning() << "sound is not supported";
     }
     
-    const std::string &profession = addressee.custom( "KADDRESSBOOK", "X-Profession" ).toStdString();
+    const std::string &profession = toStdString(addressee.custom( "KADDRESSBOOK", "X-Profession" ));
     if (!profession.empty()) {
         setCustom(profession, "X-Profession", c);
     }
     
-    const std::string &adrBook = addressee.custom( "KADDRESSBOOK", "X-AddressBook" ).toStdString();
+    const std::string &adrBook = toStdString(addressee.custom( "KADDRESSBOOK", "X-AddressBook" ));
     if (!adrBook.empty()) {
         setCustom(adrBook, "X-AddressBook", c);
     }
@@ -804,46 +807,44 @@ Kolab::Contact fromKABC(const KABC::Addressee &addressee)
 DistList fromKABC(const KABC::ContactGroup &cg)
 {
     DistList dl;
-    dl.setName(cg.name().toStdString());
-    dl.setUid(cg.id().toStdString());
+    dl.setName(toStdString(cg.name()));
+    dl.setUid(toStdString(cg.id()));
     
     std::vector <Kolab::ContactReference > members;
-    for (int i = 0; i < cg.contactReferenceCount(); i++) {
+    for (unsigned int i = 0; i < cg.dataCount(); i++) {
+        const KABC::ContactGroup::Data &data = cg.data(i);
+        members.push_back(Kolab::ContactReference(Kolab::ContactReference::EmailReference, toStdString(data.email()), toStdString(data.name())));
+    }
+    for (unsigned int i = 0; i < cg.contactReferenceCount(); i++) {
         const KABC::ContactGroup::ContactReference &ref = cg.contactReference(i);
-        if (!ref.preferredEmail().isEmpty()) {
-            members.push_back(Kolab::ContactReference(Kolab::ContactReference::EmailReference, ref.preferredEmail().toStdString())); //TODO name?
-        } else if (!ref.uid().isEmpty()) {
-            //TODO find uid of referenced contact (akonadi item id)
-            Warning() << "uid reference not yet supported";
-        } else {
-            Error() << "invalid contact reference";
-        }
+        members.push_back(Kolab::ContactReference(Kolab::ContactReference::UidReference, toStdString(ref.uid())));
+    }
+    
+    if (cg.contactGroupReferenceCount() > 0) {
+        kWarning() << "Tried to save contact group references, which should have been resolved already";
     }
     
     dl.setMembers(members);
     
-    
     return dl;
 }
 
 KABC::ContactGroup toKABC(const DistList &dl)
 {
-    KABC::ContactGroup cg(QString::fromStdString(dl.name()));
-    cg.setId(QString::fromStdString(dl.uid()));
+    KABC::ContactGroup cg(fromStdString(dl.name()));
+    cg.setId(fromStdString(dl.uid()));
     foreach(const Kolab::ContactReference &m, dl.members()) {
-        KABC::ContactGroup::ContactReference ref;
+        KABC::ContactGroup::Data data;
         switch (m.type()) {
             case Kolab::ContactReference::EmailReference:
-                ref.setPreferredEmail(QString::fromStdString(m.email()));
+                cg.append(KABC::ContactGroup::Data(fromStdString(m.name()), fromStdString(m.email())));
+                break;
             case Kolab::ContactReference::UidReference:
-                //TODO find contact based on uid, and then set the corresponding akonadi item id as uid
-                //m.uid() //TODO 
-                //ref.setUid(//akonadi item id of referenced contact);
-                Warning() << "uid reference not yet supported";
+                cg.append(KABC::ContactGroup::ContactReference(fromStdString(m.uid())));
+                break;
             default:
                 Error() << "invalid contact reference";
         }
-        cg.append(ref);
     }
     
     return cg;
@@ -851,4 +852,4 @@ KABC::ContactGroup toKABC(const DistList &dl)
 
 
     } //Namespace
-} //Namespace
\ No newline at end of file
+} //Namespace
