@@ -21,6 +21,7 @@
 #include <QtCore/QBitArray>
 #include <QtCore/QVector>
 #include <QtCore/QDebug>
+#include <QUrl>
 #include <vector>
 
 #include "kolabformat/errorhandler.h"
@@ -31,6 +32,7 @@ namespace Kolab {
 
 //The uid of a contact which refers to the uuid of a contact in the addressbook
 #define CUSTOM_KOLAB_CONTACT_UUID "X_KOLAB_CONTACT_UUID"
+#define CUSTOM_KOLAB_CONTACT_CUTYPE "X_KOLAB_CONTACT_CUTYPE"
 
 KCalCore::Duration toDuration(const Kolab::Duration &d)
 {
@@ -252,8 +254,8 @@ QString getCustomProperty(const QString &id, const T &e)
 {
     std::vector<CustomProperty> &props = e.customProperties();
     foreach (const CustomProperty &p, props) {
-        if (QString::fromStdString(p.identifier) == id) {
-            return QString::fromStdString(p.value);
+        if (fromStdString(p.identifier) == id) {
+            return fromStdString(p.value);
         }
     }
 }
@@ -262,7 +264,7 @@ template <typename T>
 void setIncidence(KCalCore::Incidence &i, const T &e)
 {
     if (!e.uid().empty()) {
-        i.setUid(QString::fromStdString(e.uid()));
+        i.setUid(fromStdString(e.uid()));
     }
     
     i.setCreated(toDate(e.created()));
@@ -275,44 +277,69 @@ void setIncidence(KCalCore::Incidence &i, const T &e)
         i.setDtStart(toDate(e.start()));
     }
 
-    i.setSummary(QString::fromStdString(e.summary())); //TODO detect richtext
-    i.setDescription(QString::fromStdString(e.description())); //TODO detect richtext
+    i.setSummary(fromStdString(e.summary())); //TODO detect richtext
+    i.setDescription(fromStdString(e.description())); //TODO detect richtext
     i.setStatus(toStatus(e.status()));
     foreach (const Kolab::Attendee a, e.attendees()) {
         /*
          * KCalCore always sets a UID if empty, but that's just a pointer, and not the uid of a real contact.
          * Since that means the semantics of the two are different, we have to store the kolab uid as a custom property.
          */
-        KCalCore::Attendee::Ptr attendee = KCalCore::Attendee::Ptr(new KCalCore::Attendee(QString::fromStdString(a.contact().name()), 
-                                                                                          QString::fromStdString(a.contact().email()),
+        KCalCore::Attendee::Ptr attendee = KCalCore::Attendee::Ptr(new KCalCore::Attendee(fromStdString(a.contact().name()), 
+                                                                                          fromStdString(a.contact().email()),
                                                                                           a.rsvp(),
                                                                                           toPartStat(a.partStat()),
                                                                                           toRole(a.role()) ));
         if (!a.contact().uid().empty()) { //TODO Identify contact from addressbook based on uid
-            attendee->customProperties().setNonKDECustomProperty(CUSTOM_KOLAB_CONTACT_UUID, QString::fromStdString(a.contact().uid()));
+            attendee->customProperties().setNonKDECustomProperty(CUSTOM_KOLAB_CONTACT_UUID, fromStdString(a.contact().uid()));
+        }
+        if (!a.delegatedTo().empty()) {
+            if (a.delegatedTo().size() > 1) {
+                WARNING("multiple delegatees are not supported");
+            }
+            attendee->setDelegate(toMailto(a.delegatedTo().front().email(), a.delegatedTo().front().name()).toString());
+        }
+        if (!a.delegatedFrom().empty()) {
+            if (a.delegatedFrom().size() > 1) {
+                WARNING("multiple delegators are not supported");
+            }
+            attendee->setDelegator(toMailto(a.delegatedFrom().front().email(), a.delegatedFrom().front().name()).toString());
+        }
+        if (!a.cutype() != CutypeIndividual) {
+            attendee->customProperties().setNonKDECustomProperty(CUSTOM_KOLAB_CONTACT_CUTYPE, QString::number(a.cutype()));
         }
         i.addAttendee(attendee);
     }
     foreach (const Kolab::Attachment a, e.attachments()) {
         KCalCore::Attachment::Ptr ptr;
         if (!a.uri().empty()) {
-            ptr = KCalCore::Attachment::Ptr(new KCalCore::Attachment(QString::fromStdString(a.uri()), QString::fromStdString(a.mimetype())));
+            ptr = KCalCore::Attachment::Ptr(new KCalCore::Attachment(fromStdString(a.uri()), fromStdString(a.mimetype())));
         } else {
-            ptr = KCalCore::Attachment::Ptr(new KCalCore::Attachment(QByteArray::fromRawData(a.data().c_str(), a.data().size()), QString::fromStdString(a.mimetype())));
+            ptr = KCalCore::Attachment::Ptr(new KCalCore::Attachment(QByteArray::fromRawData(a.data().c_str(), a.data().size()), fromStdString(a.mimetype())));
         }
         if (!a.label().empty()) {
-            ptr->setLabel(QString::fromStdString(a.label()));
+            ptr->setLabel(fromStdString(a.label()));
         }
         i.addAttachment(ptr);
     }
-    
-//     i.setCustomProperties(); //TODO
+
+    QMap<QByteArray, QString> props;
+    foreach (const Kolab::CustomProperty &prop, e.customProperties()) {
+        QString key;
+        if (prop.identifier.compare(0, 5, "X-KDE")) {
+            key.append(QString::fromLatin1("X-KOLAB-"));
+        }
+        key.append(fromStdString(prop.identifier));
+        props.insert(key.toLatin1(), fromStdString(prop.value));
+//         i.setCustomProperty("KOLAB", fromStdString(prop.identifier).toLatin1(), fromStdString(prop.value));
+    }
+    i.setCustomProperties(props);
 }
 
 template <typename T, typename I>
 void getIncidence(T &i, const I &e)
 {
-    i.setUid(e.uid().toStdString());
+    i.setUid(toStdString(e.uid()));
     i.setCreated(fromDate(e.created()));
     i.setLastModified(fromDate(e.lastModified()));
     i.setSequence(e.revision());
@@ -320,16 +347,31 @@ void getIncidence(T &i, const I &e)
     i.setCategories(fromStringList(e.categories()));
     
     i.setStart(fromDate(e.dtStart()));
-    i.setSummary(e.summary().toStdString());
-    i.setDescription(e.description().toStdString());
+    i.setSummary(toStdString(e.summary()));
+    i.setDescription(toStdString(e.description()));
     i.setStatus(fromStatus(e.status()));
     std::vector<Kolab::Attendee> attendees;
     foreach (const KCalCore::Attendee::Ptr ptr, e.attendees()) {
         const QString &uid = ptr->customProperties().nonKDECustomProperty(CUSTOM_KOLAB_CONTACT_UUID);
-        Kolab::Attendee a(Kolab::ContactReference(ptr->email().toStdString(), ptr->name().toStdString(), uid.toStdString()));
+        Kolab::Attendee a(Kolab::ContactReference(toStdString(ptr->email()), toStdString(ptr->name()), toStdString(uid)));
         a.setRSVP(ptr->RSVP());
         a.setPartStat(fromPartStat(ptr->status()));
         a.setRole(fromRole(ptr->role()));
+        if (!ptr->delegate().isEmpty()) {
+            std::string name;
+            const std::string &email = fromMailto(ptr->delegate(), name);
+            a.setDelegatedTo(std::vector<Kolab::ContactReference>() << Kolab::ContactReference(email, name));
+        }
+        if (!ptr->delegator().isEmpty()) {
+            std::string name;
+            const std::string &email = fromMailto(ptr->delegator(), name);
+            a.setDelegatedFrom(std::vector<Kolab::ContactReference>() << Kolab::ContactReference(email, name));
+        }
+        const QString &cutype = ptr->customProperties().nonKDECustomProperty(CUSTOM_KOLAB_CONTACT_CUTYPE);
+        if (!cutype.isEmpty()) {
+            a.setCutype(static_cast<Kolab::Cutype>(cutype.toInt()));
+        }
+        
         attendees.push_back(a);
     }
     i.setAttendees(attendees);
@@ -337,16 +379,22 @@ void getIncidence(T &i, const I &e)
     foreach (const KCalCore::Attachment::Ptr &ptr, e.attachments()) {
         Kolab::Attachment a;
         if (ptr->isUri()) {
-            a.setUri(ptr->uri().toStdString(), ptr->mimeType().toStdString());
+            a.setUri(toStdString(ptr->uri()), toStdString(ptr->mimeType()));
         } else {
-            a.setData(std::string(ptr->decodedData().data(), ptr->decodedData().size()), ptr->mimeType().toStdString());
+            a.setData(std::string(ptr->decodedData().data(), ptr->decodedData().size()), toStdString(ptr->mimeType()));
         }
-        a.setLabel(ptr->label().toStdString());
+        a.setLabel(toStdString(ptr->label()));
         attachments.push_back(a);
     }
     i.setAttachments(attachments);
 
-    //TODO custom properties
+    std::vector<Kolab::CustomProperty> customProperties;
+    const QMap<QByteArray, QString> &props = e.customProperties();
+    for (QMap<QByteArray, QString>::const_iterator it = props.begin(); it != props.end(); it++) {
+        QString key = QString(it.key());
+        customProperties.push_back(Kolab::CustomProperty(toStdString(key.remove("X-KOLAB-")), toStdString(it.value())));
+    }
+    i.setCustomProperties(customProperties);
 }
 
 int toWeekDay(Kolab::Weekday wday)
@@ -600,10 +648,10 @@ void setTodoEvent(KCalCore::Incidence &i, const T &e)
 {
     i.setPriority(toPriority(e.priority()));
     if (!e.location().empty()) {
-        i.setLocation(QString::fromStdString(e.location())); //TODO detect richtext
+        i.setLocation(fromStdString(e.location())); //TODO detect richtext
     }
     if (e.organizer().isValid()) {
-        i.setOrganizer(KCalCore::Person::Ptr(new KCalCore::Person(QString::fromStdString(e.organizer().name()), QString::fromStdString(e.organizer().email())))); //TODO handle uid too
+        i.setOrganizer(KCalCore::Person::Ptr(new KCalCore::Person(fromStdString(e.organizer().name()), fromStdString(e.organizer().email())))); //TODO handle uid too
     }
     if (e.recurrenceID().isValid()) {
         i.setRecurrenceId(toDate(e.recurrenceID())); //TODO THISANDFUTURE
@@ -615,17 +663,17 @@ void setTodoEvent(KCalCore::Incidence &i, const T &e)
             case Kolab::Alarm::EMailAlarm: {
                 KCalCore::Person::List receipents;
                 foreach (Kolab::ContactReference c ,a.attendees()) {
-                    KCalCore::Person::Ptr person = KCalCore::Person::Ptr(new KCalCore::Person(QString::fromStdString(c.name()), QString::fromStdString(c.email())));
+                    KCalCore::Person::Ptr person = KCalCore::Person::Ptr(new KCalCore::Person(fromStdString(c.name()), fromStdString(c.email())));
                     receipents.append(person);
                 }
-                alarm->setEmailAlarm(QString::fromStdString(a.summary()), QString::fromStdString(a.description()), receipents);
+                alarm->setEmailAlarm(fromStdString(a.summary()), fromStdString(a.description()), receipents);
             }
                 break;
             case Kolab::Alarm::DisplayAlarm:
-                alarm->setDisplayAlarm(QString::fromStdString(a.text()));
+                alarm->setDisplayAlarm(fromStdString(a.text()));
                 break;
             case Kolab::Alarm::AudioAlarm:
-                alarm->setAudioAlarm(QString::fromStdString(a.audioFile().uri()));
+                alarm->setAudioAlarm(fromStdString(a.audioFile().uri()));
                 break;
             default:
                 Error() << "invalid alarm";
@@ -652,9 +700,9 @@ template <typename T, typename I>
 void getTodoEvent(T &i, const I &e)
 {
     i.setPriority(fromPriority(e.priority()));
-    i.setLocation(e.location().toStdString());
-    if (!e.organizer()->email().isEmpty()) {
-        i.setOrganizer(Kolab::ContactReference(Kolab::ContactReference::EmailReference, e.organizer()->email().toStdString(), e.organizer()->name().toStdString())); //TODO handle uid too
+    i.setLocation(toStdString(e.location()));
+    if (e.organizer() && !e.organizer()->email().isEmpty()) {
+        i.setOrganizer(Kolab::ContactReference(Kolab::ContactReference::EmailReference, toStdString(e.organizer()->email()), toStdString(e.organizer()->name()))); //TODO handle uid too
     }
     i.setRecurrenceID(fromDate(e.recurrenceId()), false); //TODO THISANDFUTURE
     getRecurrence(i, e);
@@ -669,19 +717,19 @@ void getTodoEvent(T &i, const I &e)
         }
         switch (a->type()) {
             case KCalCore::Alarm::Display:
-                alarm = Kolab::Alarm(a->text().toStdString());
+                alarm = Kolab::Alarm(toStdString(a->text()));
                 break;
             case KCalCore::Alarm::Email: {
                 std::vector<Kolab::ContactReference> receipents;
                 foreach(const KCalCore::Person::Ptr &p, a->mailAddresses()) {
-                    receipents.push_back(Kolab::ContactReference(p->email().toStdString(), p->name().toStdString()));
+                    receipents.push_back(Kolab::ContactReference(toStdString(p->email()), toStdString(p->name())));
                 }
-                alarm = Kolab::Alarm(a->mailSubject().toStdString(), a->mailText().toStdString(), receipents);
+                alarm = Kolab::Alarm(toStdString(a->mailSubject()), toStdString(a->mailText()), receipents);
             }
                 break;
             case KCalCore::Alarm::Audio: {
                 Kolab::Attachment audioFile;
-                audioFile.setUri(a->audioFile().toStdString(), std::string());
+                audioFile.setUri(toStdString(a->audioFile()), std::string());
                 alarm = Kolab::Alarm(audioFile);
             }
                 break;
@@ -754,6 +802,12 @@ KCalCore::Todo::Ptr toKCalCore ( const Todo &todo )
     if (todo.due().isValid()) {
         e->setDtDue(toDate(todo.due()));
     }
+    if (!todo.relatedTo().empty()) {
+        e->setRelatedTo(Conversion::fromStdString(todo.relatedTo().front()), KCalCore::Incidence::RelTypeParent);
+        if (todo.relatedTo().size() > 1) {
+            Error() << "only one relation support but got multiple";
+        }
+    }
     e->setPercentComplete(todo.percentComplete());
     return e;
 }
@@ -765,6 +819,12 @@ Todo fromKCalCore ( const KCalCore::Todo &todo )
     getTodoEvent(t, todo);
     t.setDue(fromDate(todo.dtDue(true)));
     t.setPercentComplete(todo.percentComplete());
+    const QString relatedTo = todo.relatedTo(KCalCore::Incidence::RelTypeParent);
+    if (!relatedTo.isEmpty()) {
+        std::vector<std::string> relateds;
+        relateds.push_back(Conversion::toStdString(relatedTo));
+        t.setRelatedTo(relateds);
+    }
     return t;
 }
 
@@ -787,4 +847,4 @@ Journal fromKCalCore ( const KCalCore::Journal &journal )
 
 
 }
-}
\ No newline at end of file
+}
