@@ -34,7 +34,6 @@
 
 #include <kabc/addressee.h>
 #include <kcalcore/freebusyurlstore.h>
-#include <kio/netaccess.h>
 #include <kdebug.h>
 #include <QFile>
 #include <float.h>
@@ -1179,10 +1178,12 @@ QImage Contact::loadPictureFromAddressee( const KABC::Picture& picture )
   QImage img;
   if ( !picture.isIntern() && !picture.url().isEmpty() ) {
     QString tmpFile;
-    if ( KIO::NetAccess::download( picture.url(), tmpFile, 0 /*no widget known*/ ) ) {
-      img.load( tmpFile );
-      KIO::NetAccess::removeTempFile( tmpFile );
-    }
+    kWarning() << "external pictures are currently not supported";
+    //FIXME add kio support to libcalendaring or use libcurl
+//     if ( KIO::NetAccess::download( picture.url(), tmpFile, 0 /*no widget known*/ ) ) {
+//       img.load( tmpFile );
+//       KIO::NetAccess::removeTempFile( tmpFile );
+//     }
   } else
     img = picture.data();
   return img;
@@ -1193,14 +1194,14 @@ QByteArray KolabV2::Contact::loadSoundFromAddressee( const KABC::Sound& sound )
   QByteArray data;
   if ( !sound.isIntern() && !sound.url().isEmpty() ) {
     QString tmpFile;
-    if ( KIO::NetAccess::download( sound.url(), tmpFile, 0 /*no widget known*/ ) ) {
-      QFile f( tmpFile );
-      if ( f.open( QIODevice::ReadOnly ) ) {
-        data = f.readAll();
-        f.close();
-      }
-      KIO::NetAccess::removeTempFile( tmpFile );
-    }
+//     if ( KIO::NetAccess::download( sound.url(), tmpFile, 0 /*no widget known*/ ) ) {
+//       QFile f( tmpFile );
+//       if ( f.open( QIODevice::ReadOnly ) ) {
+//         data = f.readAll();
+//         f.close();
+//       }
+//       KIO::NetAccess::removeTempFile( tmpFile );
+//     }
   } else
     data = sound.data();
   return data;
