@@ -39,7 +39,6 @@
 #include <kcalcore/journal.h>
 #include <kdebug.h>
 #include <kurl.h>
-#include <kio/netaccess.h>
 
 #include <QBitArray>
 
@@ -773,8 +772,10 @@ void Incidence::setFields( const KCalCore::Incidence::Ptr &incidence )
     }
   }
 
-  Email org( incidence->organizer()->name(), incidence->organizer()->email() );
-  setOrganizer( org );
+  if (incidence->organizer()) {
+    Email org( incidence->organizer()->name(), incidence->organizer()->email() );
+    setOrganizer( org );
+  }
 
   // Attendees:
   KCalCore::Attendee::List attendees = incidence->attendees();
