@@ -48,12 +48,6 @@ class KCalConversionTest : public QObject
     void testDateTZ_data();
     void testDateTZ();
 
-    void testEventConflict_data();
-    void testEventConflict();
-    
-    void testTimesInInterval_data();
-    void testTimesInInterval();
-    void testTimesInIntervalBenchmark();
 };
 
 #endif
\ No newline at end of file
